@@ -47,22 +47,23 @@ mut-publish --version
 Environment Variables:
 MUT_CACHE_CONTROL               A value for the Cache-Control header to be attached to
                                 each file when in deploy mode. If not provided, it
-                                defaults to 8 hours (max-age=28800). Staging always
-                                uses no-cache.
+                                uses a set of defaults derived from the Gatsby documentation.
+                                Staging always uses no-cache.
 """
 
 import collections
 import concurrent.futures
+import fnmatch
 import functools
 import hashlib
+import json
 import logging
-import time
 import mimetypes
 import os
 import posixpath
 import re
 import sys
-import json
+import time
 
 import boto3
 import boto3.s3.transfer
@@ -84,6 +85,37 @@ DELETION_DANGER_THRESHOLD = 350
 T = TypeVar('T')
 
 
+class CacheControl:
+    """A manager for cache-control headers. When constructed it takes a list of stanzas,
+       each of the type Tuple[List[str], str]: this maps a set of wildcard patterns
+       to a cache-control header, following the Python fnmatch rules[1].
+
+       See the DEFAULT_PATTERN for an example.
+
+       [1]: https://docs.python.org/3/library/fnmatch.html
+    """
+    # Based on https://www.gatsbyjs.com/docs/caching/
+    DEFAULT_PATTERN = '''[
+        [["*.html", "*/page-data/*.json", "*/sw.js"],
+            "public, max-age=0, must-revalidate"
+        ],
+        [["*"], "public, max-age=28800"]
+    ]'''
+
+    def __init__(self, stanzas: List[Tuple[List[str], str]]) -> None:
+        self.stanzas: List[Tuple[List[re.Pattern[str]], str]] = [
+            ([re.compile(fnmatch.translate(pat)) for pat in stanza[0]], stanza[1]) for stanza in stanzas
+        ]
+
+    def __getitem__(self, key: str) -> str:
+        """Return a Cache-Control header value for a given key."""
+        for patterns, cache_control in self.stanzas:
+            if any(pat.match(key) for pat in patterns):
+                return cache_control
+
+        return "no-cache"
+
+
 class Timer:
     def __init__(self, name: str) -> None:
         self.name = name
@@ -126,10 +158,10 @@ def remove_beginning(beginning: str, s: str) -> str:
     return s[len(beginning):] if s.startswith(beginning) else s
 
 
-def chunks(l: List[T], n: int) -> Iterable[List[T]]:
+def chunks(data: List[T], n: int) -> Iterable[List[T]]:
     """Split a list into chunks of at most length n."""
-    for i in range(0, len(l), n):
-        yield l[i:(i + n)]
+    for i in range(0, len(data), n):
+        yield data[i:(i + n)]
 
 
 def run_pool(tasks: List[Callable[[None], None]], n_workers: int = 5, retries: int = 1) -> None:
@@ -194,7 +226,7 @@ class ChangeSet:
     def __init__(self, verbose: bool, deployed_url_prefix: str) -> None:
         self.verbose = verbose
         self.suspicious_files = []  # type: List[str]
-        self.full_deploy_urls = []  # type: List[Tuple[str, str]]
+        self.modified_full_urls = []  # type: List[Tuple[str, str]]
         self.deployed_url_prefix = deployed_url_prefix
 
         self.commands_delete = []  # type: List[Tuple[str, str]]
@@ -205,16 +237,18 @@ class ChangeSet:
             multipart_threshold=UPLOAD_CHUNK_SIZE,
             multipart_chunksize=UPLOAD_CHUNK_SIZE)
 
-        self.cache_control = "no-cache"
+        self.cache_control = CacheControl([])
 
-    def delete(self, objects: List[str], tag: str = 'D') -> None:
-        """Request deletion of a list of objects."""
-        self.commands_delete.extend((tag, x) for x in objects)
+    def delete(self, keys: List[str], flag: str = 'D') -> None:
+        """Request deletion of a list of keys."""
+        for key in keys:
+            self.commands_delete.append((flag, key))
+            self.modified_full_urls.append((flag, self.deployed_url_prefix + '/' + key))
 
-    def delete_redirects(self, objects: List[str]) -> None:
+    def delete_redirects(self, keys: List[str]) -> None:
         """Request deletion of a list of redirects. Behavior is the same as ChangeSet.delete():
            the distinction is informational for ChangeSet.print()."""
-        self.delete(objects, tag='DR')
+        self.delete(keys, flag='DR')
 
     def upload(self, path: str, key: str, new_file: bool) -> None:
         """Upload a local path into the bucket. new_file is informational for ChangeSet.print()."""
@@ -225,7 +259,7 @@ class ChangeSet:
             self.suspicious_files.append(key)
 
         # full url with deploy prefix in a separate list from the list of files to actually upload
-        self.full_deploy_urls.append((flag, self.deployed_url_prefix + '/' + key))
+        self.modified_full_urls.append((flag, self.deployed_url_prefix + '/' + key))
 
         self.commands_upload.append((flag, path, key))
 
@@ -242,7 +276,7 @@ class ChangeSet:
         # convert full urls with deploy prefix to json
         if return_json:
             json_obj = {'urls': []}  # type: Dict[str, List[str]]
-            for command in self.full_deploy_urls:
+            for command in self.modified_full_urls:
                 flag, path = command
                 json_obj['urls'].append(path)
             print(json.dumps(json_obj))
@@ -310,11 +344,22 @@ class ChangeSet:
 
     def __upload(self, s3: Any, src_path: str, key: str) -> None:
         """Thread worker helper to handle uploading a single file to S3."""
+        # Deduce a mimetype and content encoding
+        guessed_type, guessed_content_encoding = mimetypes.guess_type(src_path)
+        if guessed_type:
+            mimetype_headers: Dict[str, str] = {
+                'ContentType': guessed_type
+            }
+            if guessed_content_encoding:
+                mimetype_headers['ContentEncoding'] = guessed_content_encoding
+        else:
+            mimetype_headers = {'ContentType': 'binary/octet-stream'}
+
         try:
             # Default to 8-hour TTL
             s3.upload_file(src_path, key, ExtraArgs={
-                'ContentType': mimetypes.guess_type(src_path)[0] or 'binary/octet-stream',
-                'CacheControl': self.cache_control
+                'CacheControl': self.cache_control[key],
+                **mimetype_headers
             }, Config=self.s3_config)
             sys.stdout.write('.')
             sys.stdout.flush()
@@ -574,7 +619,7 @@ class Staging:
     @property
     def namespace(self) -> str:
         """Staging places each stage under a unique namespace computed from an
-           arbitrary username and branch This helper returns such a
+           arbitrary username and branch. This helper returns such a
            namespace, appropriate for constructing a new Staging instance."""
         # The S3 prefix for this staging site
         return '/'.join([x for x in (self.config.prefix,
@@ -669,7 +714,8 @@ class DeployStaging(Staging):
 
     def __init__(self, config: Config) -> None:
         super().__init__(config)
-        self.changes.cache_control = os.environ.get('MUT_CACHE_CONTROL', 'max-age=28800')
+        parsed_stanzas = json.loads(os.environ.get('MUT_CACHE_CONTROL', CacheControl.DEFAULT_PATTERN))
+        self.changes.cache_control = CacheControl(parsed_stanzas)
 
     @property
     def namespace(self) -> str:
